#   This file is part of Gnupylot.
#
#    Gnupylot is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Gnupylot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Gnupylot; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
Created on 20/gen/2010

@author: Lorenzo Rovigatti
'''

from PyQt4 import QtCore, QtGui
import GUI
import Gnuplot
from lib import common, paths, Plot, DataFile
import tempfile
import os
from sys import exit

class Controller(QtCore.QObject):
    '''Controller of the application. 
    Everything must pass for an instance of this class.
    It's passed to the widgets that need it as a parameter of their constructors.
    '''
    def __init__(self, parent=None, gnuplot_bin=None, debug=False, files=False):
        '''
        Constructor
        '''
        QtCore.QObject.__init__(self, parent)

        self.config = GUI.Config()
        """Current L{Config<GUI.Config>}.
        @type: L{Config<GUI.Config>}"""
        self.NAME = "Gnupylot"
        """Name of the application"""
        self.VERSION = "0.1.9"
        """Version of the application"""
        self.COMMENT = "A multiplatform front-end for gnuplot written in Python using PyQt4"
        """Description of the application"""
        self.AUTHORS = "Lorenzo Rovigatti"
        """Current contributors"""
        self.WEBSITE = "http://www.sourceforge.net/projects/gnupylot"
        """Project's website"""

        self.plots = {}
        """dictionary of L{Plot} objects containing all the plot currently in session. 
        We need a dictionary because we have to label each plot with a unique index.
        @type: list of L{Plot}s"""
        self.__currentPlot = -1
        """internal variables used to store the current plot number.
        @type: int"""
        self.__tempNames = []
        """list used internally to keep track of temp files used.
        @type: list of strings"""
        self.__nextPlotIndex = 0
        """Contains a number that will be the next plot's index.
        @type: int
        """

        common.debugFlag = debug

        self.undogroup = QtGui.QUndoGroup(self)
        """L{QUndoGroup} used to store Plots' L{QUndoStack}.
        @type: L{QUndoGroup}"""

        self.initGnuplot(gnuplot_bin)
        self.initMainWindow()

        self.newPlot(files)

        self.mainWindow.show()

        # we have to put this control here because the inizializations are too entangled... blame on me!
        if self.isValidGnuplotBin(Gnuplot.GnuplotOpts.gnuplot_command) == False:
            self.emitGUIError("Gnuplot's executable '%s' not found" % Gnuplot.GnuplotOpts.gnuplot_command)
            exit(-1)

    def initMainWindow(self):
        self.mainWindow = GUI.MainWindow(self)
        self.connect(self, QtCore.SIGNAL('plotToUpdate'), self.mainWindow.autoUpdatePlot)
        self.connect(self, QtCore.SIGNAL('GUIError(PyQt_PyObject)'), self.mainWindow.showGUIError)

        # load main window's state
        stateFile = QtCore.QFile(paths.getConfigDir() + "lastState")
        stateFile.open(stateFile.ReadOnly)
        self.mainWindow.restoreState(stateFile.readAll())
        stateFile.close()

    def initGnuplot(self, gnuplot_bin):
        if gnuplot_bin != None:
            Gnuplot.GnuplotOpts.gnuplot_command = gnuplot_bin
        else:
            Gnuplot.GnuplotOpts.gnuplot_command = self.config.config['gnuplotBin']

        # set terminal to png and set the output        
        Gnuplot.GnuplotOpts.default_term = 'png'
        self.gnuplot = Gnuplot.Gnuplot(debug=1)
        """Pipe to a L{gnuplot<GnuplotProcess>} process.
        @type: L{GnuplotProcess}"""

        # forward Gnuplot's methods
        self.set_string = self.gnuplot.set_string
        self.gnuplotSave = self.gnuplot.save
        self.gnuplotLoad = self.gnuplot.load
        self.isGnuplotError = self.gnuplot.isError
        self.gnuplotHardcopy = self.gnuplot.hardcopy
        self.setNoPause = self.gnuplot.setNoPause

    # thanks to http://stackoverflow.com/questions/377017/test-if-executable-exists-in-python
    def isValidGnuplotBin(self, gnuplotBin):
        def is_exe(fpath):
            return os.path.exists(fpath) and os.access(fpath, os.X_OK)

        fpath = os.path.split(gnuplotBin)[0]
        if fpath:
            if is_exe(gnuplotBin):
                return True
        else:
            for path in os.environ["PATH"].split(os.pathsep):
                exe_file = os.path.join(path, gnuplotBin)
                if is_exe(exe_file):
                    return True

        return False

    def getUndoGroup(self):
        return self.undogroup

    def closeAllPlots(self):
        """Close all plots."""
        self.plots = {}
        self.__currentPlot = -1

    def closePlot(self, index):
        """Delete the current plot and load another one."""
        del self.plots[index]

        if len(self.plots) == 0:
            self.newPlot()
        elif index == self.__currentPlot:
            # we need to find the plot that's under the deleted plot. If there isn't
            # then we'll use the plot above
            ks = sorted(self.plots.keys())
            for k in ks:
                if k < index: choosen = k
                else:
                    choosen = k
                    break

            self.setCurrentPlot(choosen)

    def newPlot(self, files=False):
        """Add an empty Plot to the current plots and make it the current one.
        If the optional argument is not False then load data from those files considering column 1 as x and column 2 as y.
        @param files: optional argument. If it's not False then it has to be a tuple containing filenames
        @type files: bool|tuple of filenames
        """
        nPlot = Plot.Plot(self.gnuplot)
        self.undogroup.addStack(nPlot.getUndoStack())
        self.undogroup.setActiveStack(nPlot.getUndoStack())
        self.plots[self.__nextPlotIndex] = nPlot
        self.connect(nPlot.getUndoStack(), QtCore.SIGNAL('indexChanged(int)'), self.updateCurrentPlot)
        self.connect(nPlot, QtCore.SIGNAL('titleChanged(PyQt_PyObject)'), self.mainWindow.setTitle)
        self.__currentPlot = self.__nextPlotIndex
        self.mainWindow.addNewPlotAction(self.__nextPlotIndex)
        # let's empty the gnuplot process' queue
        self.gnuplot._clear_queue()
        nPlot.reset()

        if files != False:
            dataFile = DataFile.DataFile(self)
            for f in files:
                dataFile.parseFile(f)
                # with no arguments the default behaviour is to plot first and second column
                # if there are at least 2 columns, otherwise index + first column
                dataFile.filterDatasets()
                data = dataFile.filteredDatasets
                filename = self.getTempFile()
                orig_name = f
                for i in range(0, len(data)):
                    d = data[i]
                    if i > 0: name = orig_name + " " + str(i)
                    else: name = orig_name
                    title = name

                    nPlot.data.appendData(data=d, name=name, filename=filename, title=title)
            self.updateCurrentPlot()

        self.mainWindow.loadCurrentPlot()
        self.__nextPlotIndex += 1

    # FIXME: dirty method!
    def __call__(self, s):
        """Controller calls are forwarded to the stored gp_qprocess.
        """
        self.gnuplot(s)

    def readyToExit(self):
        """Check for unsaved plots and cleans up stuff. 
        Deletes all the temp files used."""
        for ind in self.plots:
            p = self.plots[ind]
            if p.unsaved == True:
                res = QtGui.QMessageBox.question(self.mainWindow, "Unsaved plot",
                                                 "Plot '" + p.windowTitle + "' is unsaved, are you sure you want to exit Gnupylot?",
                                                 QtGui.QMessageBox.Yes | QtGui.QMessageBox.No)
                if res == QtGui.QMessageBox.No:
                    return False

        for tmpname in self.__tempNames:
            if os.path.isfile(tmpname): os.unlink(tmpname)

        # save the state to a file in the configDir
        stateFile = QtCore.QFile(paths.getConfigDir() + "lastState")
        stateFile.open(stateFile.WriteOnly)
        stateFile.write(self.mainWindow.saveState())
        stateFile.close()

        return True

    def getConfig(self):
        """
        Returns the config object.
        """
        return self.config

    def getNumberOfPlots(self):
        """
        Returns the number of open plots.
        """
        return len(self.plots)

    def getCurrentPlot(self):
        """Returns the current Plot.
        @return: L{Plot}
        """
        return self.plots[self.__currentPlot]

    def getCurrentPlotIndex(self):
        """Returns the index of the current plot.
        @return: int
        """
        return self.__currentPlot

    def setCurrentPlot(self, plot):
        """Sets the plot-th Plot as current
        
        """
        if plot not in self.plots:
            self.emitGUIError("Plot requested is non existant", "Number of plots: " + str(self.getNumberOfPlots()) + ", plot choosen: " + plot)
            return

        self.__currentPlot = plot
        self.updateCurrentPlot()
        self.mainWindow.loadCurrentPlot()

    def savePlot(self, filename=None):
        """Save the current plot."""
        plot = self.getCurrentPlot()
        plot.save(filename)
        common.debug("Saving plot to '" + plot.filename + "'")

    def loadPlot(self, filename):
        """Load a plot from the given file."""
        self.getCurrentPlot().load(filename)
        self.updateCurrentPlot()
        self.mainWindow.loadCurrentPlot()
        common.debug("Loading plot from '" + filename + "'")

    def updateCurrentPlot(self):
        """
        Push items in the Gnuplot.py's queue.
        """
        plot = self.getCurrentPlot()
        funcs = plot.getFunctions()
        data = plot.getData()
        labels = plot.getLabels()

        # tabula rasa
        self.gnuplot._clear_queue()
        self.gnuplot("unset label")

        # we want to show functions, labels and datasets that aren't hidden so we check against getHiddenStatus

        # add functions
        self.gnuplot.defineItems([[f.name, f.definition] for f in funcs if f.getHiddenStatus() == False])
        self.gnuplot._add_to_queue([f.getPlotItem() for f in funcs if f.getHiddenStatus() == False])
        # add labels
        for label in labels:
            if label.getHiddenStatus() == False: self.gnuplot(label.getPlotItem())
        # add datasets
        self.gnuplot._add_to_queue([d.getPlotItem() for d in data if d.getHiddenStatus() == False])

        self.emit(QtCore.SIGNAL('plotToUpdate'))

    def itemListEmpty(self):
        """
        Returns whetever gnuplot's itemlist is empty.
        """
        return len(self.gnuplot.itemlist) == 0

    def getTempFile(self):
        """
        Returns a valid temporary filename.
        """
        nName = tempfile.mktemp()
        self.__tempNames.append(nName)
        return nName

    def emitGUIError(self, s, appendToDebug=None):
        """
        Emits a GUIError and writes to common.debug the error.
        """
        common.debug("GUI Error: " + s, appendToDebug)
        self.emit(QtCore.SIGNAL('GUIError(PyQt_PyObject)'), s)
