#   This file is part of Gnupylot.
#
#    Gnupylot is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Gnupylot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Gnupylot; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
Created on 29/gen/2010

@author: Lorenzo Rovigatti
'''

import string
from PyQt4 import QtCore, QtGui
from UI import AddEditRemoveDock, CurveStyleBaseWidget

try:
    from lib import common, UndoCommands
except ImportError:
    from src.lib import common, UndoCommands

class BasePlotObjectContextMenu(QtGui.QMenu):
    '''
    A menu to be shown when the user right clicks on a dataset, function or label. Can be extended to provide further functionalities.
    '''
    def __init__(self, parent, clickedItemName, editCallback, removeCallback, hideCallback):
        QtGui.QMenu.__init__(self, parent)

        self.addAction("Edit %s" % clickedItemName, editCallback)
        self.addAction("Delete %s" % clickedItemName, removeCallback)
        self.addSeparator()

        self.hideAction = self.addAction("Hide %s" % clickedItemName, hideCallback)
        self.hideAction.setCheckable(True)
#        self.addAction("Hide %s" % clickedItemName, hideCallback)


class BaseAERDockWidget(QtGui.QDockWidget, AddEditRemoveDock.Ui_AERDock):
    '''
    Base class for datasets, functions and labels dock widgets.
    '''
    def __init__(self, controller, parent, modelName):
        '''
        Constructor
        '''
        QtGui.QDockWidget.__init__(self, parent)
        self.setupUi(self)

        self.controller = controller
        """Reference to the current L{Controller}.
        @type: L{Controller}"""
        self.modelName = modelName
        """String containing the name of the model (e.g. label or dataset).
        @type: String 
        """

        self.connect(self.addButton, QtCore.SIGNAL('clicked()'), self.addObject)
        self.connect(self.editButton, QtCore.SIGNAL('clicked()'), self.editObject)
        self.connect(self.delButton, QtCore.SIGNAL('clicked()'), self.removeObject)

        self.contextMenu = BasePlotObjectContextMenu(self, modelName, self.editObject, self.removeObject, self.hideObject)
        self.connect(self.itemList, QtCore.SIGNAL('customContextMenuRequested(const QPoint &)'), self.showContextMenu)

        self.setWindowTitle("%ss" % string.capitalize(self.modelName))

    def showContextMenu(self, point):
        index = self.getSelectedIndex()
        if index != -1:
            object = self.model.getObject(index)
            self.contextMenu.hideAction.setChecked(object.getHiddenStatus())
            self.contextMenu.exec_(self.cursor().pos())

    def addObject(self):
        """
        To be implemented in subclasses
        """
        pass

    def editObject(self):
        """
        To be implemented in subclasses
        """
        pass

    def removeObject(self):
        """
        To be implemented in subclasses
        """
        pass

    def hideObject(self):
        index = self.getSelectedIndex()
        if index == -1:
            self.controller.emitGUIError("Select a %s to hide" % self.modelName)
            return

        comm = UndoCommands.ToggleHiddenStatus(self.model, index)

        self.controller.getCurrentPlot().pushCommand(comm)

    def getSelectedIndex(self):
        """
        Returns the index of the selected item. Returns -1 if no item is selected
        """
        list = self.itemList.selectedIndexes()
        if len(list) == 0: return - 1

        return list[0]

class ChooseFileWidget(QtGui.QWidget):
    '''
    A widget used to select a file via a "browse" button
    '''
    def __init__(self, parent, filter, selectedFilter, name):
        QtGui.QWidget.__init__(self, parent)
        self.filter = filter
        self.selectedFilter = selectedFilter
        self.name = name

        self.filename = QtGui.QLineEdit(self)
        self.filename.setMinimumWidth(300)
        self.browse = QtGui.QPushButton("Browse...", self)
        self.connect(self.browse, QtCore.SIGNAL("clicked()"), self.showFileDialog)

        hbox = QtGui.QHBoxLayout(self)
        hbox.addWidget(self.filename)
        hbox.addWidget(self.browse)
        hbox.setContentsMargins(0, 0, 0, 0)

    def showFileDialog(self):
        filename = QtGui.QFileDialog.getOpenFileName(self, self.name, ".", self.filter, self.selectedFilter)
        if filename: self.filename.setText(filename)

    def getFilename(self):
        return str(self.filename.text()).strip()

class CurveStyleWidget(QtGui.QWidget, CurveStyleBaseWidget.Ui_CurveStyleBaseWidget):
    '''
    A widget used to pick up the line style for datasets and functions.
    '''
    def __init__(self, parent):
        QtGui.QWidget.__init__(self, parent)

        self.setupUi(self)
        self.initComboBoxes()

        self.connect(self.styleComboBox, QtCore.SIGNAL('currentIndexChanged(int)'), self.onChangeStyle)

    def forwardGettersAndSetters(self, obj):
        """
        Calls to obj.getters are forwarded to CurveStyleWidget getters
        """
        obj.getCurveStyle = self.getCurveStyle
        obj.getLineType = self.getLineType
        obj.getLineThickness = self.getLineThickness
        obj.getPointType = self.getPointType
        obj.getPointSize = self.getPointSize

        obj.setCurveStyle = self.setCurveStyle
        obj.setLineType = self.setLineType
        obj.setLineThickness = self.setLineThickness
        obj.setPointType = self.setPointType
        obj.setPointSize = self.setPointSize

    def onChangeStyle(self, index):
        """
        Enable point comboboxes if style is lines or pointslines, disabled them otherwise
        """
        if index != 0:
            self.lineTypeComboBox.setEnabled(True)
            self.lineThicknessComboBox.setEnabled(True)

            txt = self.styleComboBox.currentText()
            enabled = not (txt in ("lines", "impulses", "dots", "steps", "boxes"))
            self.pointTypeComboBox.setEnabled(enabled)
            self.pointSizeComboBox.setEnabled(enabled)
        else:
            self.toggleFieldsActivation(False, True)

    def initComboBoxes(self):
        """
        Init comboboxes for line type and thickness and point type and size.
        """
        lineTypeList = ["-"] + [str(x) for x in range(1, common.N_LINE_TYPES)]
        lineThickList = ["-"] + [str(x) for x in range(1, common.N_LINE_THICKNESSES)]
        pointTypeList = ["-"] + [str(x) for x in range(1, common.N_POINT_TYPES)]
        pointSizeList = ["-"] + [str(x) for x in range(1, common.N_POINT_TYPES)]

        self.lineTypeComboBox.addItems(lineTypeList)
        self.lineThicknessComboBox.addItems(lineThickList)
        self.pointTypeComboBox.addItems(pointTypeList)
        self.pointSizeComboBox.addItems(pointSizeList)

    def toggleFieldsActivation(self, val, keepStyle=False):
        self.pointTypeComboBox.setEnabled(val)
        self.pointSizeComboBox.setEnabled(val)
        self.lineTypeComboBox.setEnabled(val)
        self.lineThicknessComboBox.setEnabled(val)
        if keepStyle == False: self.styleComboBox.setEnabled(val)

    # getters
    def getCurveStyle(self):
        return str(self.styleComboBox.currentText())

    def getLineType(self):
        return str(self.lineTypeComboBox.currentText())

    def getLineThickness(self):
        return str(self.lineThicknessComboBox.currentText())

    def getPointType(self):
        return str(self.pointTypeComboBox.currentText())

    def getPointSize(self):
        return str(self.pointSizeComboBox.currentText())

    # setters
    def setCurveStyle(self, txt):
        nIndex = self.styleComboBox.findText(txt)
        return self.styleComboBox.setCurrentIndex(nIndex)

    def setLineType(self, txt):
        nIndex = self.lineTypeComboBox.findText(txt)
        return self.lineTypeComboBox.setCurrentIndex(nIndex)

    def setLineThickness(self, txt):
        nIndex = self.lineThicknessComboBox.findText(txt)
        return self.lineThicknessComboBox.setCurrentIndex(nIndex)

    def setPointType(self, txt):
        nIndex = self.pointTypeComboBox.findText(txt)
        return self.pointTypeComboBox.setCurrentIndex(nIndex)

    def setPointSize(self, txt):
        nIndex = self.pointSizeComboBox.findText(txt)
        return self.pointSizeComboBox.setCurrentIndex(nIndex)
