#   This file is part of Gnupylot.
#
#    Gnupylot is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Gnupylot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Gnupylot; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
Created on 20/gen/2010

@author: Lorenzo Rovigatti
'''

from PyQt4 import QtCore, QtGui
from UI import AddEditStyleData, EditData
from CommonWidgets import ChooseFileWidget, CurveStyleWidget, BaseAERDockWidget

try:
    from lib import PlotObjects, UndoCommands, DataFile
except ImportError:
    from src.lib import PlotObjects, UndoCommands, DataFile
from copy import deepcopy

class DatasetDockWidget(BaseAERDockWidget):
    """A dockbar widget used to manage gnuplot's numeric data."""
    def __init__(self, controller, parent):
        BaseAERDockWidget.__init__(self, controller, parent, "dataset")

        self.setupEditMenu()

    def setupEditMenu(self):
        self.editMenu = QtGui.QMenu(self)
        self.editMenu.addAction("Data", self.editObjectData)
        self.editMenu.addAction("Appearance", self.editObjectAppearance)

    def loadObjectModel(self):
        self.model = self.controller.getCurrentPlot().data
        """Stores the data currently defined. 
        @type: L{DataModel}
        """
        self.itemList.setModel(self.model)

    def editObject(self):
        # we need to pass to menu's exec_() the position where we want it to be placed (at the right of editButton)
        self.editMenu.exec_(self.editButton.mapToGlobal(QtCore.QPoint(0, 0) + QtCore.QPoint(self.editButton.size().width(), 0)))

    def addObject(self):
        """Called when the user click on the 'Add' button.
        It opens a dialog used to add data to the current plot.
        """
        dialog = AddDataFormDialog(self.controller, self)
        if dialog.exec_() == dialog.Accepted:
            data, args = dialog.getDatasetsValues()

            name = args['name']
            title = args['title']
            # append all the sets
            for i in range(0, len(data)):
                # the first set use the given name, the other ones use name + i
                if i > 0:
                    nName = name + str(i)
                    # default value for the title is the name
                    if len(title) > 0: args['title'] = title + str(i)
                    else: args['title'] = nName
                else:
                    nName = name
                    if len(title) > 0: args['title'] = title
                    else: args['title'] = nName

                args['filename'] = self.controller.getTempFile()
                args['name'] = nName
                args['data'] = data[i]

                comm = UndoCommands.AddDatasetCommand(self.model, args)

                self.controller.getCurrentPlot().pushCommand(comm)

    def editObjectData(self):
        """Called when the user click on the 'Edit -> Data' button or uses the context menu.
        It opens a dialog used to edit the currently selected dataset.
        """
        dialog = EditDataFormDialog(self.controller, self)

        index = self.getSelectedIndex()
        if index == -1:
            self.controller.emitGUIError("Select a dataset to edit")
            return

        data = deepcopy(self.model.getObject(index))
        dialog.setData(data)

        if dialog.exec_() == dialog.Accepted:
            comm = UndoCommands.EditDatasetCommand(self.model, index, data)
            self.controller.getCurrentPlot().pushCommand(comm)

    def editObjectAppearance(self):
        """Called when the user click on the 'Edit -> Appearance' button or uses the context menu.
        It opens a dialog used to edit the currently selected dataset.
        """
        index = self.getSelectedIndex()
        if index == -1:
            self.controller.emitGUIError("Select a dataset to edit")
            return

        dialog = EditStyleDataFormDialog(self.controller, self)
        data = deepcopy(self.model.getObject(index))

        # load dataset's data
        dialog.setDatasetName(data.name)
        if data.isParameter("title"): dialog.setDatasetTitle(data.getParameter('title'))
        if data.isParameter("with_"): dialog.setCurveStyle(data.getParameter("with_"))
        if data.isParameter("linetype"): dialog.setLineType(data.getParameter("linetype"))
        if data.isParameter("linewidth"): dialog.setLineThickness(data.getParameter("linewidth"))
        if data.isParameter("pointtype"): dialog.setPointType(data.getParameter("pointtype"))
        if data.isParameter("pointsize"): dialog.setPointSize(data.getParameter("pointsize"))

        if dialog.exec_() == dialog.Accepted:
            args = dialog.getDatasetsValues()
            comm = UndoCommands.EditDatasetAppearanceCommand(self.model, index, data, args)
            self.controller.getCurrentPlot().pushCommand(comm)

    def removeObject(self):
        """Called when the user click on the 'Del' button.
        It shows a popup asking for a confirm. If it's accepted than the selected dataset 
        will be deleted.
        """
        index = self.getSelectedIndex()
        if index == -1:
            self.controller.emitGUIError("Select a dataset to remove")
            return

        res = QtGui.QMessageBox.question(self, "Confirmation",
                                         "Are you sure you want to completely delete '" + \
                                         self.model.data(index) + "'?",
                                         QtGui.QMessageBox.Yes | QtGui.QMessageBox.No,
                                         QtGui.QMessageBox.Yes)

        if res == QtGui.QMessageBox.Yes:
            comm = UndoCommands.DelDatasetCommand(self.model, index)
            self.controller.getCurrentPlot().pushCommand(comm)


class EditStyleDataFormDialog(QtGui.QDialog, AddEditStyleData.Ui_AddEditStyleData):
    def __init__(self, controller, parent):
        QtGui.QDialog.__init__(self, parent)
        self.controller = controller
        """Reference to the current L{Controller}.
        @type: L{Controller}"""

        self.setupUi(self)
        self.setWindowTitle("Edit appearance")

        # add a ChooseFileWidget by hand
        self.chooseFile = ChooseFileWidget(self, "All files(*)", "*", "Choose datafile")
        self.formlayout.insertRow(0, "&File", self.chooseFile)

        # if the title line edit gets the focus and it contains no text then
        # insert the name as a default value
        import types
        def onTitleFocus(self, event=None):
            if len(self.text()) == 0:
                self.setText(self.parent().datasetName.text())
                self.selectAll()
            QtGui.QLineEdit.focusInEvent(self, event)
        # dynamically bind onTitleFocus to self.datasetTitle
        f = types.MethodType(onTitleFocus, self.datasetTitle, QtGui.QLineEdit)
        self.datasetTitle.focusInEvent = f

        self.styleWidget = CurveStyleWidget(self)
        self.styleWidget.forwardGettersAndSetters(self)
        self.styleWidget.toggleFieldsActivation(False, True)
        self.styleWidget.horizontalLayout.insertSpacing(0, 32)

        self.formlayout.insertRow(6, self.styleWidget)

        # disabled file + x/y/err widgets
        self.chooseFile.setEnabled(False)
        self.xexpr.setEnabled(False)
        self.xfrom.setEnabled(False)
        self.yexpr.setEnabled(False)
        self.yfrom.setEnabled(False)
        self.errexpr.setEnabled(False)
        self.errfrom.setEnabled(False)

        self.setFixedSize(self.sizeHint())

    def getDatasetsValues(self):
        """
        Retrieve all the datasets-related selected values.
        @return: a data, args tuple
        """
        title = self.getDatasetTitle()
        name = self.getDatasetName()
        style = self.getCurveStyle()
        lt = self.getLineType()
        lw = self.getLineThickness()
        pt = self.getPointType()
        ps = self.getPointSize()

        args = {"name" : name, "title" : title}
        # check if there is an error bar too
        if style != "-": args['with_'] = style
        # linetype
        if lt != "-": args['with_'] += " linetype " + lt
        # linewidth
        if lw != "-": args['with_'] += " linewidth " + lw
        # pointtype
        if pt != "-": args['with_'] += " pointtype " + pt
        # pointsize
        if ps != "-": args['with_'] += " pointsize " + ps

        return args

    def getDatasetTitle(self):
        return str(self.datasetTitle.text())

    def getDatasetName(self):
        return str(self.datasetName.text())

    def setDatasetTitle(self, val):
        self.datasetTitle.setText(val)

    def setDatasetName(self, val):
        self.datasetName.setText(val)


class AddDataFormDialog(QtGui.QDialog, AddEditStyleData.Ui_AddEditStyleData):
    """Add data dialog."""
    def __init__(self, controller, parent):
        QtGui.QDialog.__init__(self, parent)
        self.controller = controller
        """Reference to the current L{Controller}.
        @type: L{Controller}"""

        self.setupUi(self)
        # add a ChooseFileWidget by hand
        self.chooseFile = ChooseFileWidget(self, "All files(*)", "*", "Choose datafile")
        self.formlayout.insertRow(0, "&File", self.chooseFile)

        self.connect(self.chooseFile.filename, QtCore.SIGNAL('textChanged(const QString &)'),
                     self.readFile)
        self.connect(self.errfrom, QtCore.SIGNAL('currentIndexChanged(int)'), self.onSelectErrorIndex)
        self.connect(self.errexpr, QtCore.SIGNAL('textChanged(const QString &)'), self.onChangeErrorExpr)

        # if the title line edit gets the focus and it contains no text then
        # insert the name as a default value
        import types
        def onTitleFocus(self, event=None):
            if len(self.text()) == 0:
                self.setText(self.parent().datasetName.text())
                self.selectAll()
            QtGui.QLineEdit.focusInEvent(self, event)
        # dynamically bind onTitleFocus to self.datasetTitle
        f = types.MethodType(onTitleFocus, self.datasetTitle, QtGui.QLineEdit)
        self.datasetTitle.focusInEvent = f

        self.styleWidget = CurveStyleWidget(self)
        self.styleWidget.forwardGettersAndSetters(self)
        self.styleWidget.horizontalLayout.insertSpacing(0, 32)

        self.formlayout.insertRow(6, self.styleWidget)

        self.setFixedSize(self.sizeHint())
        self.toggleFieldsActivation(False)

        self.dataFile = DataFile.DataFile(self)

    def onSelectErrorIndex(self, index):
        """Slot activated when an error index is selected. If an index is choosen then it disables 
        styleComboBox because style must be set to 'errorbars'.
        """
        if index != 0:
            self.styleWidget.styleComboBox.setCurrentIndex(7)
            self.styleWidget.styleComboBox.setEnabled(False)
        else:
            self.styleWidget.styleComboBox.setEnabled(True)

    def onChangeErrorExpr(self, text):
        """Slot activated when something is wrote in errexpr. If the length of the expression in > 0 then it disables 
        styleComboBox because style must be set to 'errorbars'.
        """
        if len(text) != 0:
            self.styleWidget.styleComboBox.setCurrentIndex(7)
            self.styleWidget.styleComboBox.setEnabled(False)
        else:
            self.styleWidget.styleComboBox.setEnabled(True)

    def toggleFieldsActivation(self, val):
        self.datasetName.setEnabled(val)
        self.datasetTitle.setEnabled(val)
        self.xfrom.setEnabled(val)
        self.xexpr.setEnabled(val)
        self.yfrom.setEnabled(val)
        self.yexpr.setEnabled(val)
        self.errfrom.setEnabled(val)
        self.errexpr.setEnabled(val)
        if val == True:
            self.styleWidget.styleComboBox.setEnabled(True)
        else:
            self.styleWidget.toggleFieldsActivation(val)

    def readFile(self, filename):
        """Parse the file using a {DataFile} object and fill the column controls. The parsed data is in L{self.dataFile}.availableDatasets"""
        self.xfrom.clear()
        self.yfrom.clear()
        self.errfrom.clear()

        try:
            cols = self.dataFile.parseFile(filename)
        except Exception, e:
            self.controller.emitGUIError("File '" + filename + "' has an incorrect format", e)
            self.toggleFieldsActivation(False)
            return

        if cols == False:
            self.toggleFieldsActivation(False)
            return

        # fill x, y, and err selectboxes with the available columns
        self.xfrom.addItems(["ind"] + cols)
        self.yfrom.addItems(["ind"] + cols)
        self.errfrom.addItems(["-"] + cols)

        if len(cols) > 0:
            self.xfrom.setCurrentIndex(1)
            if len(cols) > 1:
                self.yfrom.setCurrentIndex(2)
            else: self.yfrom.setCurrentIndex(1)

        self.toggleFieldsActivation(True)

    def getDatasetsValues(self):
        """
        Retrieve all the datasets-related selected values.
        @return: a data, args tuple
        """
        data = self.getFilteredDatasets()
        title = self.getDatasetTitle()
        name = self.getDatasetName()
        style = self.getCurveStyle()
        lt = self.getLineType()
        lw = self.getLineThickness()
        pt = self.getPointType()
        ps = self.getPointSize()

        args = {"name" : name, "title" : title}
        # check if there is an error bar too
        if data[0].shape[1] == 3:
            args['with_'] = 'errorbars'
        # otherwise use the style selected
        else:
            if style != "-": args['with_'] = style
        # linetype
        if lt != "-": args['with_'] += " linetype " + lt
        # linewidth
        if lw != "-": args['with_'] += " linewidth " + lw
        # pointtype
        if pt != "-": args['with_'] += " pointtype " + pt
        # pointsize
        if ps != "-": args['with_'] += " pointsize " + ps

        return data, args

    def getFilteredDatasets(self):
        return self.dataFile.filteredDatasets

    def getCols(self):
        """Returns a tuple containing the choosen file columns (or expressions involving file columns) for each axis."""
        cols = []
        # x
        xexpr = self.xexpr.text()
        if len(xexpr) > 0: cols.append(str(xexpr))
        else:
            cols.append("$" + str(self.xfrom.currentText()))
        # y
        yexpr = self.yexpr.text()
        if len(yexpr) > 0: cols.append(str(yexpr))
        else:
            cols.append("$" + str(self.yfrom.currentText()))
        # err
        errexpr = self.errexpr.text()
        if len(errexpr) > 0: cols.append(str(errexpr))
        else:
            if self.errfrom.currentText() != '-':
                cols.append("$" + str(self.errfrom.currentText()))

        return cols

    def getDatasetTitle(self):
        return str(self.datasetTitle.text())

    def getDatasetName(self):
        return str(self.datasetName.text())

    def setDataTitle(self, val):
        self.datasetTitle.setText(val)

    def setDataName(self, val):
        self.datasetName.setText(val)

    def accept(self):
        if len(self.getDatasetName()) == 0:
            self.controller.emitGUIError("Data name can't be empty")
            return

        try:
            self.dataFile.filterDatasets(self.getCols())
        except Exception, e:
            self.controller.emitGUIError("Invalid expression", e)
            return

        return QtGui.QDialog.accept(self)


class EditDataFormDialog(QtGui.QDialog, EditData.Ui_EditData):
    """Edit data dialog."""
    def __init__(self, controller, parent):
        QtGui.QDialog.__init__(self, parent)
        self.controller = controller
        """Reference to the current L{Controller}.
        @type: L{Controller}"""

        self.setupUi(self)

        self.connect(self.insertRowButton, QtCore.SIGNAL('clicked()'), self.insertRow)
        self.connect(self.deleteRowsButton, QtCore.SIGNAL('clicked()'), self.deleteRows)

    def getSortedRows(self):
        indexes = self.dataView.selectedIndexes()

        if len(indexes) != 0:
            rowList = []
            for i in indexes:
                if i.row() not in rowList:
                    rowList.append(i.row())
            rowList.sort()
            rowList.reverse()
        else: rowList = []

        return rowList


    def insertRow(self):
        # if there is any selection, insert a new row before the first selected item
        # otherwise append the row at the end
        rowList = self.getSortedRows()

        if len(rowList) == 0: nPos = self.dataset.rowCount()
        else: nPos = rowList[0]

        self.dataset.insertRow(nPos)


    def deleteRows(self):
        rows = self.getSortedRows()

        if len(rows) != 0:
            self.dataset.removeSparseRows(rows)
            self.dataView.clearSelection()
        else:
            self.controller.emitGUIError("Select the row(s) you want to delete")

    def setData(self, data):
        self.dataset = data
        self.dataView.setModel(data)
        self.dataView.setItemDelegate(PlotObjects.DataDelegate())

    def accept(self):
        QtGui.QDialog.accept(self)

