#   This file is part of Gnupylot.
#
#    Gnupylot is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Gnupylot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Gnupylot; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
Created on 20/gen/2010

@author: Lorenzo Rovigatti
'''

from PyQt4 import QtGui
from UI import AddEditFunction
from CommonWidgets import CurveStyleWidget, BaseAERDockWidget

try:
    from lib import UndoCommands
except ImportError:
    from src.lib import UndoCommands

class FuncDockWidget(BaseAERDockWidget):
    """A dockbar widget used to manage gnuplot's functions."""
    def __init__(self, controller, parent):
        '''
        Constructor
        '''
        BaseAERDockWidget.__init__(self, controller, parent, "function")

    def loadObjectModel(self):
        self.model = self.controller.getCurrentPlot().functions
        """Stores the functions currently defined. 
        @type: L{FuncModel}
        """
        self.itemList.setModel(self.model)

    def addObject(self):
        """Called when the user click on the 'Add' button.
        It opens a dialog used to add a function to the current plot.
        """
        dialog = FunctionFormDialog("Add", self)
        if dialog.exec_() == dialog.Accepted:
            args = dialog.getFunctionDialogValues()

            comm = UndoCommands.AddFunctionCommand(self.model, args)
            self.controller.getCurrentPlot().pushCommand(comm)

    def editObject(self):
        """Called when the user click on the 'Edit' button.
        It opens a dialog used to edit the currently selected function.
        """
        index = self.getSelectedIndex()
        if index == -1:
            self.controller.emitGUIError("Select a function to edit")
            return

        dialog = FunctionFormDialog("Edit", self)

        func = self.model.getObject(index)
        plotItem = func.getPlotItem()

        # load function's data
        dialog.setFunctionName(func.name[:-3])
        dialog.setFunctionDefinition(func.definition)
        dialog.setFunctionTitle(plotItem.get_option('title'))
        if func.isParameter("with_"): dialog.setCurveStyle(func.getParameter("with_"))
        if func.isParameter("linetype"): dialog.setLineType(func.getParameter("linetype"))
        if func.isParameter("linewidth"): dialog.setLineThickness(func.getParameter("linewidth"))
        if func.isParameter("pointtype"): dialog.setPointType(func.getParameter("pointtype"))
        if func.isParameter("pointsize"): dialog.setPointSize(func.getParameter("pointsize"))

        if dialog.exec_() == dialog.Accepted:
            args = dialog.getFunctionDialogValues()

            comm = UndoCommands.EditFunctionCommand(self.model, index, args)
            self.controller.getCurrentPlot().pushCommand(comm)

    def removeObject(self):
        """Called when the user click on the 'Del' button.
        It shows a popup asking for a confirm. If it's accepted than the selected function 
        will be deleted.
        """
        index = self.getSelectedIndex()
        if index == -1:
            self.controller.emitGUIError("Select a function to remove")
            return

        res = QtGui.QMessageBox.question(self, "Confirmation",
                                         "Are you sure you want to completely delete '" + \
                                         self.model.data(index) + "' function?",
                                         QtGui.QMessageBox.Yes | QtGui.QMessageBox.No,
                                         QtGui.QMessageBox.Yes)

        if res == QtGui.QMessageBox.Yes:
            comm = UndoCommands.DelFunctionCommand(self.model, index)
            self.controller.getCurrentPlot().pushCommand(comm)


class FunctionFormDialog(QtGui.QDialog, AddEditFunction.Ui_AddEditFunction):
    """Add/Edit functions dialog."""
    def __init__(self, role, parent):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        self.setWindowTitle(role + " function")

        # if the title line edit gets the focus and it contains no text then
        # insert the name as a default value
        import types
        def onTitleFocus(self, event=None):
            if len(self.text()) == 0:
                self.setText(self.parent().nameLineEdit.text())
                self.selectAll()
            QtGui.QLineEdit.focusInEvent(self, event)
        # dynamically bind onTitleFocus to self.titleLineEdit
        f = types.MethodType(onTitleFocus, self.titleLineEdit, QtGui.QLineEdit)
        self.titleLineEdit.focusInEvent = f

        self.styleWidget = CurveStyleWidget(self)
        self.styleWidget.forwardGettersAndSetters(self)
        self.styleWidget.toggleFieldsActivation(False, True)
        self.styleWidget.horizontalLayout.insertSpacing(0, 1)

        self.formlayout.insertRow(3, self.styleWidget)

        self.setFixedSize(self.sizeHint())

    def getFunctionTitle(self):
        return str(self.titleLineEdit.text())

    def getFunctionName(self):
        return str(self.nameLineEdit.text() + '(x)')

    def getFunctionDefinition(self):
        return str(self.definitionLineEdit.text())

    def setFunctionTitle(self, val):
        self.titleLineEdit.setText(val)

    def setFunctionName(self, val):
        self.nameLineEdit.setText(val)

    def setFunctionDefinition(self, val):
        self.definitionLineEdit.setText(val)

    def getFunctionDialogValues(self):
        """
        Retrieve all the function-related selected values.
        """
        title = self.getFunctionTitle()
        func = self.getFunctionDefinition()
        name = self.getFunctionName()
        style = self.getCurveStyle()
        lt = self.getLineType()
        lw = self.getLineThickness()
        pt = self.getPointType()
        ps = self.getPointSize()

        args = {'name' : name, 'func' : func}
        # default value for the title is the name
        if len(title) > 0:
            args['title'] = title
        else: args['title'] = name

        if style != "-": args['with_'] = style
        # linetype
        if lt != "-": args['with_'] += " linetype " + lt
        # linewidth
        if lw != "-": args['with_'] += " linewidth " + lw
        # pointtype
        if pt != "-": args['with_'] += " pointtype " + pt
        # pointsize
        if ps != "-": args['with_'] += " pointsize " + ps

        return args

    def accept(self):
        """Checks for required fields and, if there are no errors, closes the dialog."""
        if len(self.getFunctionName()) == 0:
            QtGui.QMessageBox.warning(self, 'Error', "Function name can't be empty")
            return
        if len(self.getFunctionDefinition()) == 0:
            QtGui.QMessageBox.warning(self, 'Error', "Function definition can't be empty")
            return

        return QtGui.QDialog.accept(self)
