#   This file is part of Gnupylot.
#
#    Gnupylot is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Gnupylot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Gnupylot; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
Created on 14/gen/2010

@author: lorenzo
'''

from PyQt4 import QtGui, QtCore

class GnuplotDockWidget(QtGui.QDockWidget):
    '''
    Dockwidget containing a command line for gnuplot and a qtextedit to show
    commands, output and errors from gnuplot. Mainly useful for debug.
    '''
    def __init__(self, controller, parent):
        '''
        controller must be an instance of Controller and parent should be a
        QMainApplication
        '''
        QtGui.QDockWidget.__init__(self, parent)
        self.controller = controller
        self.setWindowTitle("Gnuplot command line")
        self.setObjectName("GnuplotCommandLine")
        
        self.gwidget = GnuplotWidget(controller, self)
        self.setWidget(self.gwidget)
        # connect parent's updatePlot to the plotToUpdate signal
        # sent by Gnuplot widget
        self.connect(self.gwidget, QtCore.SIGNAL("plotToUpdate"), parent.autoUpdatePlot)
        
    def setFocusOnCommandLine(self):
        self.gwidget.line.setFocus()
        
    def showEvent(self, event):
        self.setFocusOnCommandLine()
        
        
class GnuplotCommandLine(QtGui.QLineEdit):
    def __init__(self, parent=None):
        QtGui.QLineEdit.__init__(self, parent)
        
        self.commandHistory = []
        self.historyIndex = 0
        
    def addCommandToHistory(self, s):
        if len(self.commandHistory) > 0 and self.commandHistory[-1] == s: return
        
        self.commandHistory.append(s)
        self.historyIndex = len(self.commandHistory)
        
    def keyPressEvent(self, event):
        if event.key() == QtCore.Qt.Key_Up:
            self.historyIndex = self.historyIndex - 1
            if self.historyIndex >= 0:
                self.setText(self.commandHistory[self.historyIndex])
            else:
                self.historyIndex = 0
        elif event.key() == QtCore.Qt.Key_Down:
            self.historyIndex = self.historyIndex + 1
            if self.historyIndex < len(self.commandHistory):
                self.setText(self.commandHistory[self.historyIndex])
            else:
                self.setText("")
                self.historyIndex = len(self.commandHistory)
        else:
            QtGui.QLineEdit.keyPressEvent(self, event)


class GnuplotHistoryWidget(QtGui.QTextEdit):
    def __init__(self, controller, parent):
        QtGui.QTextEdit.__init__(self, parent)
        self.setFontFamily("DejaVu Sans Mono")
        
        self.config = controller.getConfig()
        self.errorColor = QtGui.QColor(self.config.config['gnuplotWidgetErrorColor'])
        self.outputColor = QtGui.QColor(self.config.config['gnuplotWidgetOutputColor'])
        self.commandColor = QtGui.QColor(self.config.config['gnuplotWidgetCommandColor'])
        
    def showCommand(self, s):
        self.appendOutputText("gnuplot> " + s)
        
    def addOutput(self, s, isError):
        """Add gnuplot's output to the widget text."""
        if isError == True: self.setErrorColor()
        else: self.setOutputColor()
        
        self.appendOutputText(s)
        self.setNormalColor()
        
    def appendOutputText(self, s):
        """Add s to the widget text."""
        self.setReadOnly(False)
        self.insertPlainText(s)
        self.setReadOnly(True)
        self.moveCursor(QtGui.QTextCursor.End)
        
    def setErrorColor(self):
        self.setTextColor(self.errorColor)
        
    def setOutputColor(self):
        self.setTextColor(self.outputColor)
        
    def setNormalColor(self):
        self.setTextColor(self.commandColor)


class GnuplotWidget(QtGui.QWidget):
    def __init__(self, controller, parent):
        QtGui.QWidget.__init__(self, parent)

        self.history = GnuplotHistoryWidget(controller, self)
        self.history.setReadOnly(True)
        
        self.lineLabel = QtGui.QLabel()
        self.lineLabel.setText("gnuplot> ")
        self.line = GnuplotCommandLine()
        self.connect(self.line, QtCore.SIGNAL('returnPressed()'), self.sendLine)
        
        self.cbPlot = QtGui.QCheckBox('Replot after command', self)
        self.plotAfterCommand = False
        self.connect(self.cbPlot, QtCore.SIGNAL('stateChanged(int)'), self.togglePlotAfterCommand)
        
        hbox = QtGui.QHBoxLayout()
        hbox.addWidget(self.lineLabel)
        hbox.addWidget(self.line)
        hbox.addWidget(self.cbPlot)

        vbox = QtGui.QVBoxLayout()        
        vbox.addWidget(self.history)
        vbox.addLayout(hbox)
        
        self.setLayout(vbox)
        
        self.controller = controller
        self.connect(self.controller.gnuplot, QtCore.SIGNAL('readyReadGnuplotOutput'), self.showOutput)
        self.connect(self.controller.gnuplot, QtCore.SIGNAL('readyReadGnuplotError'), self.showOutput)
        self.connect(self.controller.gnuplot, QtCore.SIGNAL('wroteCommand'), self.history.showCommand)
        
    def togglePlotAfterCommand(self, value):
        self.plotAfterCommand = self.cbPlot.isChecked()
        self.line.setFocus()
        
    def showOutput(self, s):
        self.history.addOutput(str(s), self.controller.isGnuplotError(s))
        
    def sendLine(self):
        s = str(self.line.text())
        if len(s.strip()) == 0: return
        self.line.addCommandToHistory(s)
        self.controller(s)
        
        self.line.setText("")
        self.line.setFocus()
        
        if self.plotAfterCommand: self.emit(QtCore.SIGNAL("plotToUpdate"))
        