#   This file is part of Gnupylot.
#
#    Gnupylot is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Gnupylot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Gnupylot; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
Created on 20/gen/2010

@author: Lorenzo Rovigatti
'''

from PyQt4 import QtCore, QtGui
from UI import AddEditLabel
from CommonWidgets import BaseAERDockWidget

try:
    from lib import UndoCommands
except ImportError:
    from src.lib import UndoCommands

class LabelDockWidget(BaseAERDockWidget):
    """A dockbar widget used to manage gnuplot's labels."""
    def __init__(self, controller, parent):
        '''
        Constructor
        '''
        BaseAERDockWidget.__init__(self, controller, parent, "label")

    def loadObjectModel(self):
        self.model = self.controller.getCurrentPlot().labels
        """Stores the labels currently defined. 
        @type: L{LabelModel}
        """
        self.itemList.setModel(self.model)

    def addObject(self):
        """Called when the user click on the 'Add' button.
        It opens a dialog used to add a label to the current plot.
        """
        dialog = LabelFormDialog("Add", self)
        if dialog.exec_() == dialog.Accepted:
            args = dialog.getLabelDialogValues()
            args['name'] = self.model.getNextTag()

            comm = UndoCommands.AddLabelCommand(self.model, args)
            self.controller.getCurrentPlot().pushCommand(comm)

    def editObject(self):
        """Called when the user click on the 'Edit' button.
        It opens a dialog used to edit the currently selected label.
        """
        index = self.getSelectedIndex()
        if index == -1:
            self.controller.emitGUIError("Select a label to edit")
            return

        dialog = LabelFormDialog("Edit", self)

        label = self.model.getObject(index)
        dialog.setLabelText(label.text)
        if label.font != None: dialog.setLabelFont(label.font)
        if label.position != None: dialog.setLabelPosition(label.position)

        if dialog.exec_() == dialog.Accepted:
            args = dialog.getLabelDialogValues()
            args['name'] = label.name

            comm = UndoCommands.EditLabelCommand(self.model, index, args)
            self.controller.getCurrentPlot().pushCommand(comm)

    def removeObject(self):
        """Called when the user click on the 'Del' button.
        It shows a popup asking for a confirm. If it's accepted than the selected label 
        will be deleted.
        """
        index = self.getSelectedIndex()
        if index == -1:
            self.controller.emitGUIError("Select a label to remove")
            return

        res = QtGui.QMessageBox.question(self, "Confirmation",
                                         "Are you sure you want to completely delete '" + \
                                         self.model.data(index) + "' label?",
                                         QtGui.QMessageBox.Yes | QtGui.QMessageBox.No,
                                         QtGui.QMessageBox.Yes)

        if res == QtGui.QMessageBox.Yes:
            comm = UndoCommands.DelLabelCommand(self.model, index)
            self.controller.getCurrentPlot().pushCommand(comm)


class LabelFormDialog(QtGui.QDialog, AddEditLabel.Ui_AddEditLabel):
    """Add/Edit functions dialog."""
    def __init__(self, role, parent):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        self.setWindowTitle(role + " label")
        self.fontSizeComboBox.setCurrentIndex(4)

        self.setFixedSize(self.size())

    def getLabelText(self):
        return str(self.textLineEdit.text())

    def getLabelPosition(self):
        """
        Returns a dictionary containing the specified positions:
        x -> x position
        y -> y position
        relative -> relative position with respect to x, y coordinates
        origin -> origin from which the coordinates should be computed
        """
        posX = str(self.posXLineEdit.text())
        posY = str(self.posYLineEdit.text())

        if len(posX) == 0: posX = 0
        if len(posY) == 0: posY = 0

        return {"x" : posX, "y" : posY, "relative" : str(self.posComboBox.currentText()), "origin" : str(self.posOriginComboBox.currentText()).split()[0]}

    def getLabelFont(self):
        if self.fontLineEdit.isEnabled():
            return '%s,%s' % (self.fontLineEdit.text(), self.fontSizeComboBox.currentText())
        else: return None

    def setLabelText(self, val):
        self.textLineEdit.setText(val)

    def setLabelFont(self, val):
        params = val.split(",")
        self.fontLineEdit.setText(params[0])
        self.fontSizeComboBox.setCurrentIndex(self.fontSizeComboBox.findText(params[1]))

    def setLabelPosition(self, val):
        if val != None:
            self.posXLineEdit.setText(val['x'])
            if val['y'] != None: self.posYLineEdit.setText(val['y'])
            self.posComboBox.setCurrentIndex(self.posComboBox.findText(val['relative']))
            self.posOriginComboBox.setCurrentIndex(self.posOriginComboBox.findText(val['origin'], QtCore.Qt.MatchStartsWith))

    def getLabelDialogValues(self):
        """
        Retrieve all the function-related selected values.
        """
        text = self.getLabelText()
        position = self.getLabelPosition()
        font = self.getLabelFont()

        args = {'text' : text, 'position' : position, 'font' : font}

        return args

    def accept(self):
        """Checks for required fields and, if there are no errors, closes the dialog."""
        if len(self.getLabelText()) == 0:
            QtGui.QMessageBox.warning(self, 'Error', "Label's text can't be empty")
            return
        if len(self.fontLineEdit.text()) == 0:
            QtGui.QMessageBox.warning(self, 'Warning', "Font not specified, using 'Verdana'")

        return QtGui.QDialog.accept(self)
