#   This file is part of Gnupylot.
#
#    Gnupylot is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Gnupylot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Gnupylot; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
Created on 10/gen/2010

@author: Lorenzo Rovigatti
'''
from PyQt4 import QtGui, QtCore
import time

import resources
from GnuplotDockWidget import GnuplotDockWidget
from FuncDockWidget import FuncDockWidget
from DatasetDockWidget import DatasetDockWidget
from LabelDockWidget import LabelDockWidget
from SettingsDockWidget import SettingsDockWidget
import ExportPlot

class MainWindow(QtGui.QMainWindow):
    """The application's main window."""
    INITIALIMAGE = ":/images/initial.png"
    """Stores the resource path for the initial image.
    Just a utility variable used as a constant.
    """

    def __init__(self, controller):
        """
        Constructor.
        @param controller: an application L{Controller}
        @type controller: L{Controller}
        """
        QtGui.QMainWindow.__init__(self)

        self.controller = controller
        """Reference to the current L{Controller}.
        @type: L{Controller}"""
        self.config = controller.getConfig()
        """Current L{Config<GUI.Config>}.
        @type: L{Config<GUI.Config>}"""

        self.setDockOptions(self.AnimatedDocks | self.ForceTabbedDocks)

        # connect gnuplot (hinted) errors with self.showGnuplotError
        self.connect(self.controller.gnuplot, QtCore.SIGNAL('readyReadGnuplotError'), self.showGnuplotError)

        self.plot = PlotWidget(self)
        """The gnuplot's plot is painted on this widget.
        @type: L{PlotWidget}"""

        self.setCentralWidget(self.plot)

        self.addDockbars()

        self.action_list = {}
        """This dictionary will contain all the application's actions. Contains <name, action> pairs.
        @type: Dict"""
        self.initActions()

        # init the menu
        self.setMenuBar(MenuBar(self, self.action_list))

        self.toolbar = Toolbar(self, self.action_list, self.config)
        """Main window's toolbar.
        @type: L{QToolBar}"""
        self.addToolBar(self.toolbar)

        self.toggleChangeableDockbars(False)

        self.setWindowIcon(QtGui.QIcon(':/images/logo.png'))

        # load initial image
        self.plot.setPlotPixmap(self.INITIALIMAGE)
        # tmp file to hold the background plot
        self.plotFile = controller.getTempFile()
        """Name of the temporary file that will store the plot shown."""

    def setTitle(self, title):
        self.menuBar().openPlotsGroup.checkedAction().setText(title)
        self.setWindowTitle(title + " - Gnupylot")

    def addDockbars(self):
        """Initializes and adds all the dockbars to L{self.dockbars}"""
        self.dockbars = {}
        """List of the dockbars.
        @type: list of L{QDockWidget}s."""
        self.dockbars['gDock'] = GnuplotDockWidget(self.controller, self)
        self.dockbars['gDock'].setVisible(False)
        self.dockbars['funcDock'] = FuncDockWidget(self.controller, self)
        self.dockbars['datasetDock'] = DatasetDockWidget(self.controller, self)
        self.dockbars['labelDock'] = LabelDockWidget(self.controller, self)
        self.dockbars['settingsDock'] = SettingsDockWidget(self.controller, self)
        # sets the tab position at the top of tabbed dockwidgets
        self.setTabPosition(QtCore.Qt.AllDockWidgetAreas, QtGui.QTabWidget.North)
        self.addDockWidget(QtCore.Qt.BottomDockWidgetArea, self.dockbars['gDock'])
        self.addDockWidget(QtCore.Qt.LeftDockWidgetArea, self.dockbars['funcDock'])
        self.addDockWidget(QtCore.Qt.LeftDockWidgetArea, self.dockbars['datasetDock'])
        self.addDockWidget(QtCore.Qt.LeftDockWidgetArea, self.dockbars['labelDock'])
        self.addDockWidget(QtCore.Qt.RightDockWidgetArea, self.dockbars['settingsDock'])
        self.tabifyDockWidget(self.dockbars['labelDock'], self.dockbars['funcDock'])
        self.tabifyDockWidget(self.dockbars['funcDock'], self.dockbars['datasetDock'])

    def showGnuplotError(self, message, lastCommand):
        """Opens a L{QMessageBox} showing the last gnuplot error occurred and the last command issued.
        @param message: error message
        @type message: string
        @param lastCommand: string identified as the last issued command
        @type lastCommand: string"""
        error = QtCore.QString("An error occurred. This could be related to the last issued command: '" + lastCommand.rstrip('\n') + "'")
        msg = QtGui.QMessageBox(QtGui.QMessageBox.Critical, "Gnuplot error", error, QtGui.QMessageBox.Ok, self)
        msg.setDetailedText("The error given was:\n" + message.rstrip('\n'))
        msg.open()

    def showGUIError(self, message):
        """Shows an error popup displaying an error.
        @param message: error message to be displayed
        @type message: string"""
        error = QtCore.QString(message)
        QtGui.QMessageBox.warning(self, "Error", error)

    def setReplotAfterChanges(self, val):
        """Sets the replotAfterChanges option.
        If the value is true then the plot will be update after each change.
        @param val: new value for the option
        @type val: bool"""
        self.config.config['replotAfterChanges'] = val

    def autoUpdatePlot(self):
        """Performs an L{updatePlot} if replotAfterChanges is set to True."""
        if self.config.config['replotAfterChanges'] == "1":
            self.updatePlot()

    def updatePlot(self):
        """Update the plot."""
        if self.controller.itemListEmpty() == False:
            self.controller.gnuplotHardcopy(self.plotFile, 'png')
            self.plot.setPlotPixmap(self.plotFile)

        else: self.plot.setPlotPixmap(self.INITIALIMAGE)

    def center(self):
        """Center the window on the screen."""
        screen = QtGui.QDesktopWidget().screenGeometry()
        size = self.geometry()
        self.move((screen.width() - size.width()) / 2, (screen.height() - size.height()) / 2)

    def maximize(self):
        """Maximize the window. It may *not* work on other OSes but linux."""
        screen = QtGui.QDesktopWidget().screenGeometry()
        self.setGeometry(0, 0, screen.width(), screen.height())

    def closeEvent(self, event):
        """Does some cleanup."""
        if self.controller.readyToExit() == True: event.accept()
        else: event.ignore()

    def initActions(self):
        """Initializes application's actions."""
        self.action_list['new'] = QtGui.QAction(QtGui.QIcon(':/icons/new.png'), '&New', self)
        self.action_list['new'].setShortcut('Ctrl+N')
        self.connect(self.action_list['new'], QtCore.SIGNAL('triggered()'), self.controller.newPlot)

        self.action_list['save'] = QtGui.QAction(QtGui.QIcon(':/icons/save.png'), '&Save', self)
        self.action_list['save'].setShortcut('Ctrl+S')
        self.connect(self.action_list['save'], QtCore.SIGNAL('triggered()'), self.save)

        self.action_list['saveas'] = QtGui.QAction('Save as...', self)
        self.connect(self.action_list['saveas'], QtCore.SIGNAL('triggered()'), self.saveas)

        self.action_list['load'] = QtGui.QAction(QtGui.QIcon(':/icons/open.png'), '&Load', self)
        self.action_list['load'].setShortcut('Ctrl+L')
        self.connect(self.action_list['load'], QtCore.SIGNAL('triggered()'), self.load)

        self.action_list['close'] = QtGui.QAction(QtGui.QIcon(':/icons/close.png'), '&Close', self)
        self.action_list['close'].setShortcut('Ctrl+C')
        self.connect(self.action_list['close'], QtCore.SIGNAL('triggered()'), self.closePlot)

        self.action_list['closeAll'] = QtGui.QAction('Close all', self)
        self.connect(self.action_list['closeAll'], QtCore.SIGNAL('triggered()'), self.closeAllPlots)

        self.action_list['gnuplotsave'] = QtGui.QAction("Gnuplot's save", self)
        self.connect(self.action_list['gnuplotsave'], QtCore.SIGNAL('triggered()'), self.gnuplotSave)

        self.action_list['gnuplotload'] = QtGui.QAction("Gnuplot's load", self)
        self.connect(self.action_list['gnuplotload'], QtCore.SIGNAL('triggered()'), self.gnuplotLoad)

        self.action_list['exit'] = QtGui.QAction(QtGui.QIcon(':/icons/exit.png'), '&Exit', self)
        self.action_list['exit'].setShortcut('Ctrl+Q')
        self.connect(self.action_list['exit'], QtCore.SIGNAL('triggered()'), QtCore.SLOT('close()'))

        self.action_list['replot'] = QtGui.QAction(QtGui.QIcon(':/icons/plot.png'), '&Update plot', self)
        self.action_list['replot'].setShortcut('F5')
        self.connect(self.action_list['replot'], QtCore.SIGNAL('triggered()'), self.updatePlot)

        self.action_list['undo'] = self.controller.getUndoGroup().createUndoAction(self)
        self.action_list['undo'].setIcon(QtGui.QIcon(':/icons/undo.png'))
        self.action_list['undo'].setShortcut('Ctrl+Z')
        self.action_list['redo'] = self.controller.getUndoGroup().createRedoAction(self)
        self.action_list['redo'].setIcon(QtGui.QIcon(':/icons/redo.png'))
        self.action_list['redo'].setShortcut('Ctrl+Y')

        self.action_list['replotAfterChanges'] = QtGui.QAction('&Update after changes', self)
        self.action_list['replotAfterChanges'].setCheckable(True)
        self.action_list['replotAfterChanges'].setChecked(bool(self.config.config['replotAfterChanges']))
        self.connect(self.action_list['replotAfterChanges'], QtCore.SIGNAL('triggered(bool)'),
                     self.setReplotAfterChanges)

        self.action_list['PNGExport'] = QtGui.QAction('&PNG', self)
        # to avoid the definition of a function for each export type we can just use
        # self.exportPlot for each export using lambda expressions
        callback = lambda: self.exportPlot('png')
        self.connect(self.action_list['PNGExport'], QtCore.SIGNAL('triggered()'), callback)

        self.action_list['PSExport'] = QtGui.QAction('&PostScript', self)
        self.action_list['PSExport'].setShortcut('Ctrl+P')
        callback = lambda: self.exportPlot('postscript')
        self.connect(self.action_list['PSExport'], QtCore.SIGNAL('triggered()'), callback)

        self.action_list['addFunction'] = QtGui.QAction('&Function', self)
        self.connect(self.action_list['addFunction'], QtCore.SIGNAL('triggered()'),
                     self.dockbars['funcDock'].addObject)
        self.action_list['addData'] = QtGui.QAction('&Data', self)
        self.connect(self.action_list['addData'], QtCore.SIGNAL('triggered()'),
                     self.dockbars['datasetDock'].addObject)
        self.action_list['addData'] = QtGui.QAction('&Label', self)
        self.connect(self.action_list['addData'], QtCore.SIGNAL('triggered()'),
                     self.dockbars['labelDock'].addObject)

        self.action_list['toggleChangeableDockbars'] = QtGui.QAction('&Unlock dockbars', self)
        self.action_list['toggleChangeableDockbars'].setCheckable(True)
        self.connect(self.action_list['toggleChangeableDockbars'], QtCore.SIGNAL('triggered(bool)'),
                     self.toggleChangeableDockbars)

        self.action_list['toggleFuncDock'] = self.dockbars['funcDock'].toggleViewAction()
        self.action_list['toggleFuncDock'].setShortcut('Ctrl+Alt+F')
        self.action_list['toggleDatasetDock'] = self.dockbars['datasetDock'].toggleViewAction()
        self.action_list['toggleDatasetDock'].setShortcut('Ctrl+Alt+D')
        self.action_list['toggleLabelDock'] = self.dockbars['labelDock'].toggleViewAction()
        self.action_list['toggleLabelDock'].setShortcut('Ctrl+Alt+L')
        self.action_list['toggleGDock'] = self.dockbars['gDock'].toggleViewAction()
        self.action_list['toggleGDock'].setShortcut('Ctrl+Alt+C')
        self.action_list['toggleSettingsDock'] = self.dockbars['settingsDock'].toggleViewAction()
        self.action_list['toggleSettingsDock'].setShortcut('Ctrl+Alt+S')

        self.action_list['aboutQT'] = QtGui.QAction('About &QT', self)
        self.connect(self.action_list['aboutQT'], QtCore.SIGNAL('triggered()'), QtGui.QApplication.aboutQt)

        self.action_list['about'] = QtGui.QAction(QtGui.QIcon(':/icons/about.png'), '&About Gnupylot', self)
        self.connect(self.action_list['about'], QtCore.SIGNAL('triggered()'), self.showAbout)

        self.action_list['openPlots'] = {}

    def addNewPlotAction(self, index):
        new_action = QtGui.QAction(self)
        new_action.setData(index)
        new_action.setCheckable(True)
        new_action.setVisible(True)
        new_action.setEnabled(True)
        new_action.setChecked(True)
        self.connect(new_action, QtCore.SIGNAL('triggered()'), self.setCurrentPlot)

        self.menuBar().addPlotAction(new_action)

        self.action_list['openPlots'][index] = new_action

    def setCurrentPlot(self):
        """Set the current plot. It's connected to the openPlots actions"""
        action = self.sender()
        plotNumber = action.data().toInt()[0]
        self.controller.setCurrentPlot(plotNumber)

    def loadCurrentPlot(self):
        """Makes the dockbars load all the stuff from the current plot."""
        self.dockbars['settingsDock'].loadSettings()
        self.dockbars['funcDock'].loadObjectModel()
        self.dockbars['datasetDock'].loadObjectModel()
        self.dockbars['labelDock'].loadObjectModel()
        self.updatePlot()

    def saveas(self):
        """Saves the current plot to a file, chosen via a file dialog."""
        filename = QtGui.QFileDialog.getSaveFileName(self, "Save to", "", "Gnupylot files (*.gyp);; All files (*.*)", "*.gyp")
        if filename:
            self.controller.savePlot(filename)

    def save(self):
        """Saves the current plot to a file. It calls saveas if the plot wasn't never saved before."""
        plot = self.controller.getCurrentPlot()
        if plot.filename == None:
            self.saveas()
        else: self.controller.savePlot()

    # TODO: we need a warning if the current plot is not saved and we need to choose what happens when you load a file: it goes to 
    # a new plot or it overwrites the current one?
    def load(self):
        """Loads a plot."""
        filename = QtGui.QFileDialog.getOpenFileName(self, "Load from", "", "Gnupylot files (*.gyp);; All files (*.*)", "*.gyp")
        if filename:
            try:
                self.controller.loadPlot(filename)
            except Exception as e:
                self.showGUIError("Selected file is not a valid Gnupylot save file")

    # TODO: we need a warning if the current plot is not saved
    def closePlot(self):
        """Close the current plot."""
        plotIndex = self.controller.getCurrentPlotIndex()
        self.controller.closePlot(plotIndex)
        self.menuBar().removePlotAction(self.action_list['openPlots'][plotIndex])
        # check the new current plot's action
        plotIndex = self.controller.getCurrentPlotIndex()
        self.menuBar().checkPlotAction(self.action_list['openPlots'][plotIndex])

    # TODO: we need a warning if any of the current plots is not saved
    def closeAllPlots(self):
        """Close all the open plots and load a new one"""
        self.controller.closeAllPlots()
        for a in self.action_list['openPlots']:
            self.menuBar().removePlotAction(self.action_list['openPlots'][a])

        self.controller.newPlot()

        # check the new current plot's action
        plotIndex = self.controller.getCurrentPlotIndex()
        self.menuBar().checkPlotAction(self.action_list['openPlots'][plotIndex])

    def gnuplotSave(self):
        """Saves the current plot with the gnuplot command 'save'. Currently quite useless."""
        filename = QtGui.QFileDialog.getSaveFileName(self, "Gnuplot's save", ".", "Gnuplot files (*.gp);; All files (*.*)", "*.gp")
        if filename:
            self.controller.gnuplotSave(filename)

    def gnuplotLoad(self):
        """Loads a plot from file with the gnuplot command 'load'. Currently quite useless."""
        filename = QtGui.QFileDialog.getOpenFileName(self, "Gnuplot's load", ".", "Gnuplot files (*.gp);; All files (*.*)", "*.gp")
        if filename:
            self.controller.gnuplotLoad(filename)

    def exportPlot(self, terminal):
        """Shows a dialog to export the plot to a file using the given terminal type.
        @param terminal: type of gnuplot terminal (png, postscript currently supported).
        @type terminal: string"""
        dialog = ExportPlot.ExportPlot(terminal, self.controller, self)
        if dialog.exec_() == dialog.Accepted:
            keyw = dialog.getKeyw()
            self.controller.gnuplotHardcopy(**keyw)

    def showAbout(self):
        """Shows an about dialog."""
        text = '<center><h2>' + self.controller.NAME + ' ' + self.controller.VERSION + '</h2>' + \
                self.controller.COMMENT + '<br/><br/>' + \
                '<small>Developed by ' + self.controller.AUTHORS + '<br/><br/></small>' + \
                '<a href="' + self.controller.WEBSITE + '">' + self.controller.WEBSITE + '</a>' + \
                '</center>'

        QtGui.QMessageBox.about(self, "About " + self.controller.NAME, text)

    def toggleChangeableDockbars(self, enable):
        """Enable or disable dockbar's capacity of being float, closed and resized."""
        for name, dock in self.dockbars.items():
            if enable == False: dock.setFeatures(dock.NoDockWidgetFeatures)
            else: dock.setFeatures(dock.AllDockWidgetFeatures)
            # actions have the first letter of dockbar's name capitalized
            self.action_list['toggle' + name[0].capitalize() + name[1:]].setEnabled(enable)


class PlotWidget(QtGui.QLabel):
    """A L{QLabel} whose surface is painted with the current plot."""
    def __init__(self, parent=None):
        QtGui.QLabel.__init__(self, parent)

        self.resize(640, 480)
        self.setAlignment(QtCore.Qt.AlignCenter)
        self.plotPixmap = QtGui.QPixmap()
        """L{QPixmap} containing the plot image.
        """
        self.setPixmap(self.plotPixmap)

        self.setStyleSheet('QLabel { background-color: white }')

    def setPlotPixmap(self, filename):
        """Tries to set the plot image with the given file. 
        It tries 10 times, if it doesn't succeed it shows an error message.
        @param filename: filename whose contents are to be plotted
        @type filename: string"""
        # try for 10 times (1 second total) before giving up
        counter = 10
        while not self.plotPixmap.load(filename) and counter > 0:
            time.sleep(0.1)
            counter = counter - 1

        if counter == 0: self.setText("Error while reading the plot png, try again.")
        else: self.setPixmap(self.plotPixmap)


class MenuBar(QtGui.QMenuBar):
    """Main window's L{QMenubar}"""
    def __init__(self, parent, action_list):
        QtGui.QMenuBar.__init__(self, parent)
        self.openPlotsGroup = QtGui.QActionGroup(self)
        """QActionGroup containing the actions to switch between open plots.
        @type: L{QActionGroup}: 
        """

        self.fill(action_list)

    def fill(self, action_list):
        """Initializes the menus with the actions."""
        file = self.addMenu("&File")
        file.addAction(action_list['new'])
        file.addAction(action_list['load'])
        file.addSeparator()
        file.addAction(action_list['close'])
        file.addAction(action_list['closeAll'])
        file.addSeparator()
        file.addAction(action_list['save'])
        file.addAction(action_list['saveas'])
        file.addSeparator()
        file.addAction(action_list['gnuplotsave'])
        file.addAction(action_list['gnuplotload'])
        file.addSeparator()
        file.addAction(action_list['exit'])
        self.__file = file

        plot = self.addMenu("&Plot")
        plot.addAction(action_list['replot'])
        plot.addAction(action_list['replotAfterChanges'])
        plot.addSeparator()
        plot.addAction(action_list['undo'])
        plot.addAction(action_list['redo'])
        plot.addSeparator()
        self.__plot = plot

        export = plot.addMenu("&Export as")
        export.addAction(action_list['PNGExport'])
        export.addAction(action_list['PSExport'])
        self.__export = export

        data = self.addMenu("&Data")
        add = data.addMenu("&Add")
        add.addAction(action_list['addFunction'])
        add.addAction(action_list['addData'])
        self.__data = data

        view = self.addMenu("&View")
        view.addAction(action_list['toggleChangeableDockbars'])
        view.addSeparator()
        view.addAction(action_list['toggleFuncDock'])
        view.addAction(action_list['toggleDatasetDock'])
        view.addAction(action_list['toggleLabelDock'])
        view.addAction(action_list['toggleSettingsDock'])
        view.addAction(action_list['toggleGDock'])
        view.addSeparator()
        self.__view = view

        help = self.addMenu("&Help")
        help.addAction(action_list['aboutQT'])
        help.addAction(action_list['about'])

    def addPlotAction(self, a):
        """Uncheck current plot action, if there is any, and add a new plot action.
        @param a: action to be added
        @type a: L{QAction}
        """
        # uncheck previous plot's action
        if self.openPlotsGroup.checkedAction() != None:
            self.openPlotsGroup.checkedAction().setChecked(False)

        self.openPlotsGroup.addAction(a)
        self.__view.addAction(a)

    def removePlotAction(self, a):
        """Remove the given plot action from the list.
        @param a: action to be removed
        @type a: L{QAction}
        """
        self.__view.removeAction(a)
        self.openPlotsGroup.removeAction(a)

    def checkPlotAction(self, a, check=True):
        """Check or uncheck the given action. The default behaviour is to check.
        @param a: action to be removed
        @type a: L{QAction}
        @param check: check or uncheck?
        @type check: bool
        """
        a.setChecked(check)


class Toolbar(QtGui.QToolBar):
    """Main window's L{QToolBar} containing a list of actions."""
    def __init__(self, parent, action_list, config):
        """Constructor.
        @param parent: parent of the toolbar (usually the L{MainWindow})
        @type parent: QWidget
        @param action_list: a list of actions
        @type action_list: list
        @param config: L{Config} storing at the key 'actionsInToolbar' the chosen actions to be added to the toolbar
        @type config: L{Config}
        """
        QtGui.QToolBar.__init__(self, parent)

        self.setObjectName("mainToolbar")
        self.setWindowTitle("Gnupylot toolbar")
        self.setIconSize(QtCore.QSize(24, 24))

        for action in config.config['actionsInToolbar'].split(','):
            if action == "separator": self.addSeparator()
            else: self.addAction(action_list[action])
