#   This file is part of Gnupylot.
#
#    Gnupylot is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Gnupylot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Gnupylot; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
Created on 05/feb/2010

@author: Lorenzo Rovigatti
'''

from PyQt4 import QtCore, QtGui
import common, PlotItemModel
from copy import deepcopy
from os import path
import cPickle

class Plot(QtCore.QObject):
    '''
    This class incapsulates a plot. It can be saved or loaded from file.
    It's used by the controller.
    '''
    _plot_aval_settings = {'title' : lambda gp, title : gp.title(title),
                           'xrange' : lambda gp, range : gp.set_range('xrange', range),
                           'yrange' : lambda gp, range : gp.set_range('yrange', range),
                           'xlabel' : lambda gp, label : gp.xlabel(label),
                           'ylabel' : lambda gp, label : gp.ylabel(label),
                           'xlogscale' : lambda gp, set : gp.logscale("x", set),
                           'ylogscale' : lambda gp, set : gp.logscale("y", set)
                           }

    _default_settings = {'title' : "",
                         'xrange' : ["", ""],
                         'yrange' : ["", ""],
                         'xlabel' : "",
                         'ylabel' : "",
                         'xlogscale' : False,
                         'ylogscale' : False
                         }

    def __init__(self, gnuplot_pipe=None):
        '''    
        Constructor
        '''
        QtCore.QObject.__init__(self)

        self.undostack = QtGui.QUndoStack(self)
        """L{QUndoStack} for this plot. It's added to the controller's QUndoGroup
        @type: L{QUndoStack}"""
        self.connect(self.undostack, QtCore.SIGNAL('indexChanged(int)'), lambda: self.setUnsaved(True))
        self.gp_pipe = gnuplot_pipe
        """Pipe to the current gnuplot process.
        @type: L{GnuplotProcess}"""

        self.reset()

    def emitTitleChanged(self):
        mainTitle = self.windowTitle
        if self.unsaved == True:
            mainTitle += "*"

        self.emit(QtCore.SIGNAL('titleChanged(PyQt_PyObject)'), mainTitle)

    def getFilename(self):
        return self.__filename

    def getLabels(self):
        return self.__labels

    def getFunctions(self):
        return self.__functions

    def getSettings(self):
        return self.__settings

    def getData(self):
        return self.__data

    def getTitle(self):
        return self.__windowTitle

    def isUnsaved(self):
        return self.__unsaved

    def setUnsaved(self, value):
        self.__unsaved = value
        self.emitTitleChanged()

    def setFilename(self, value):
        self.__filename = str(value)
        self.windowTitle = path.basename(self.__filename)

    def setLabels(self, value):
        self.__labels = value

    def setFunctions(self, value):
        self.__functions = value

    def setData(self, value):
        self.__data = value

    def setTitle(self, value):
        self.__windowTitle = value
        self.emitTitleChanged()

    def setSettings(self, value):
        self.gp_pipe.setNoPause(True)

        for opt, setting in value.items():
            try:
                setter = self._plot_aval_settings[opt]
            except:
                common.debug("Specified setting '" + str(opt) + "' not implemented")
                return

            self.__settings[opt] = setting
            if self.gp_pipe != None:
                setter(self.gp_pipe, setting)

        self.gp_pipe.setNoPause(False)

    def getUndoStack(self):
        return self.undostack

    def pushCommand(self, command):
        """Push a command to the undo stack."""
        self.undostack.push(command)

    def reset(self):
        """
        Resets all the L{PlotObject}s and all the settings.
        """
        self.__settings = deepcopy(self._default_settings)
        self.__functions = PlotItemModel.FuncModel()
        self.__data = PlotItemModel.DataModel()
        self.__labels = PlotItemModel.LabelModel()
        self.__unsaved = False
        self.__filename = None
        self.__windowTitle = "Untitled"

        self.emitTitleChanged()

    def save(self, filename=None):
        """Save the plot to a file. 
        If filename is None self.filename will be used, otherwise 
        self.filename will be updated with the new name and the plot will be saved there."""
        if filename != None: self.filename = filename

        with open(self.filename, "wb") as f:
            cPickle.dump(self.settings, f, protocol= -1)
            cPickle.dump(self.functions.getObjects(), f, protocol= -1)
            cPickle.dump(self.data.getObjects(), f, protocol= -1)
            cPickle.dump(self.labels.getObjects(), f, protocol= -1)

        self.unsaved = False

    def load(self, filename):
        """Load settings and data from a file with the given filename."""
        with open(filename, "rb") as f:
            self.settings = cPickle.load(f)
            self.functions = PlotItemModel.FuncModel(cPickle.load(f))
            self.data = PlotItemModel.DataModel(cPickle.load(f))
            self.labels = PlotItemModel.LabelModel(cPickle.load(f))

        # we have to manually update the files containing datasets
        for d in self.data:
            d.updateFileItem()

        self.filename = filename
        self.unsaved = False

    windowTitle = property(getTitle, setTitle, doc="Title of the plot. It's the basename of L{self.filename}")
    unsaved = property(isUnsaved, setUnsaved, doc="True if the plot is unsaved, false otherwise. @type: BooleanType")
    filename = property(getFilename, setFilename, doc="String containing the name of the file associated to this plot.")
    settings = property(getSettings, setSettings, doc="Dictionary containing <setting : value> pairs.")
    functions = property(getFunctions, setFunctions, doc="L{FuncModel} storing this plot's gnuplot functions. @type: L{FuncModel}")
    data = property(getData, setData, doc="L{DataModel} storing this plot's gnuplot numeric data. @type: L{DataModel}")
    labels = property(getLabels, setLabels, doc="To be implemented")
