#   This file is part of Gnupylot.
#
#    Gnupylot is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Gnupylot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Gnupylot; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
Created on 21/gen/2010

@author: Lorenzo Rovigatti
'''

from PyQt4 import QtCore
import PlotObjects as PO

class PlotItemModel(QtCore.QAbstractListModel):
    '''A model to be used in conjunction with a ListView. 
    It implements the methods that make possibile to use it as an iterator.
    '''
    def __init__(self, items=None, parent=None):
        '''
        Constructor
        '''
        QtCore.QAbstractListModel.__init__(self, parent)

        if items != None:
            self.objects = items
            """List containing all the PlotObjects associated with this model"""
        else:
            self.objects = []

        self.count = 0
        """Internal counter used in iterator methods"""

    # iterator methods
    def next(self):
        if self.count >= len(self.objects):
            self.count = 0
            raise StopIteration
        else:
            c = self.count
            self.count += 1
            return self.objects[c]

    def __getitem__(self, index):
        return self.objects[index.row()]

    def __iter__(self):
        return self

    def rowCount(self, parent=QtCore.QModelIndex()):
        return len(self.objects)

    def flags(self, index):
        if not index.isValid():
            return QtCore.Qt.ItemIsEnabled

        return QtCore.QAbstractItemModel.flags(self, index) | QtCore.Qt.ItemIsEditable

    def data(self, index, role=QtCore.Qt.DisplayRole):
        if not index.isValid:
            return QtCore.QVariant()

        try:
            if role == QtCore.Qt.DisplayRole:
                return self.objects[index.row()].name
        except IndexError:
            return QtCore.QVariant()

        return QtCore.QVariant()

    def insertRow(self, row):
        self.beginInsertRows(QtCore.QModelIndex(), row, row)

        self.objects.insert(row, None)

        self.endInsertRows()
        return True

    def insertRows(self, row, count):
        self.beginInsertRows(QtCore.QModelIndex(), row, row + count - 1)

        for i in range(0, count):
            self.objects.insert(row, None)

        self.endInsertRows()
        return True

    def removeRow(self, row):
        self.beginRemoveRows(QtCore.QModelIndex(), row, row)

        del self.objects[row]

        self.endRemoveRows()
        return True

    def removeRows(self, row, count):
        self.beginRemoveRows(QtCore.QModelIndex(), row, row + count - 1)

        for i in range(0, count):
            del self.objects[row]

        self.endRemoveRows()
        return True

    def getObject(self, index):
        '''Returns the list containing the name and the item at the given index.'''
        return self.objects[index.row()]

    def getObjects(self):
        """Returns all the objects stored."""
        return self.objects

    def removeItem(self, index):
        self.removeRow(index.row())

    def setData(self, index, value, role=QtCore.Qt.EditRole):
        if index.isValid() and role == QtCore.Qt.EditRole:
            self.objects[index.row()] = value
            self.emit(QtCore.SIGNAL('dataChanged(const QModelIndex &, const QModelIndex &)'), index, index)
            return True

        return False

    def insertData(self, index, value):
        """Insert the given value at the position given by index."""
        self.insertRow(index.row())
        self.setData(index, value)


class LabelModel(PlotItemModel):
    '''A model representing gnuplot's labels.'''
    def __init__(self, labels=None, parent=None):
        PlotItemModel.__init__(self, labels, parent)

        self.__maxtag = 0
        '''
        This is the highest tag used for labels. It's used to give the tag to the next label
        @type: int
        '''
        for obj in self.objects:
            if int(obj.name) > self.__maxtag: self.__maxtag = int(obj.name)

    def getNextTag(self):
        return str(self.__maxtag + 1)

    def appendLabel(self, name, text, **keyw):
        """Append a new L{Data} PlotObject and returns the assigned index.
        @param name: it must be an integer. It's the 'tag' field used in gnuplot. 
        @type name: string
        """
        last = len(self.objects)
        self.beginInsertRows(QtCore.QModelIndex(), last, last)

        self.objects.append(PO.Label(name, text, **keyw))
        index = self.createIndex(last, 0)
        self.emit(QtCore.SIGNAL('dataChanged(const QModelIndex &, const QModelIndex &)'), index, index)

        self.endInsertRows()
        return index

    def editLabel(self, index, name, text, **keyw):
        self.objects[index.row()] = PO.Label(name, text, **keyw)
        self.emit(QtCore.SIGNAL('dataChanged(const QModelIndex &, const QModelIndex &)'), index, index)

    def data(self, index, role=QtCore.Qt.DisplayRole):
        if not index.isValid:
            return QtCore.QVariant()

        try:
            if role == QtCore.Qt.DisplayRole:
                return self.objects[index.row()].text
            elif role == QtCore.Qt.ToolTipRole:
                return self.objects[index.row()].text
        except IndexError:
            return QtCore.QVariant()

        return QtCore.QVariant()


class DataModel(PlotItemModel):
    '''A model representing numeric data.'''

    def __init__(self, data=None, parent=None):
        PlotItemModel.__init__(self, data, parent)

    def appendData(self, name, data, **keyw):
        """Append a new L{Data} PlotObject and returns the assigned index."""
        last = len(self.objects)
        self.beginInsertRows(QtCore.QModelIndex(), last, last)

        self.objects.append(PO.Data(name, data, **keyw))
        index = self.createIndex(last, 0)
        self.emit(QtCore.SIGNAL('dataChanged(const QModelIndex &, const QModelIndex &)'), index, index)

        self.endInsertRows()
        return index

    def editData(self, index, name, data, **keyw):
        self.objects[index.row()] = PO.Data(name, data, **keyw)
        self.emit(QtCore.SIGNAL('dataChanged(const QModelIndex &, const QModelIndex &)'), index, index)

    def data(self, index, role=QtCore.Qt.DisplayRole):
        if not index.isValid:
            return QtCore.QVariant()

        try:
            if role == QtCore.Qt.DisplayRole:
                return self.objects[index.row()].name
            elif role == QtCore.Qt.ToolTipRole:
                return self.objects[index.row()].name
        except IndexError:
            return QtCore.QVariant()

        return QtCore.QVariant()


class FuncModel(PlotItemModel):
    '''A model representing a function in gnuplot.'''
    def __init__(self, funcs=None, parent=None):
        PlotItemModel.__init__(self, funcs, parent)

    def appendFunc(self, name, func, **keyw):
        """Append a new L{Func} PlotItem and returns the assigned index."""
        last = len(self.objects)
        self.beginInsertRows(QtCore.QModelIndex(), last, last)

        self.objects.append(PO.Function(name, func, **keyw))
        index = self.createIndex(last, 0)
        self.emit(QtCore.SIGNAL('dataChanged(const QModelIndex &, const QModelIndex &)'), index, index)

        self.endInsertRows()
        return index

    def editFunc(self, index, name, func, **keyw):
        self.objects[index.row()] = PO.Function(name, func, **keyw)
        self.emit(QtCore.SIGNAL('dataChanged(const QModelIndex &, const QModelIndex &)'), index, index)

    def data(self, index, role=QtCore.Qt.DisplayRole):
        if not index.isValid:
            return QtCore.QVariant()

        try:
            if role == QtCore.Qt.DisplayRole:
                return self.objects[index.row()].name
            elif role == QtCore.Qt.ToolTipRole:
                return self.objects[index.row()].definition
        except IndexError:
            return QtCore.QVariant()

        return QtCore.QVariant()
