#   This file is part of Gnupylot.
#
#    Gnupylot is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Gnupylot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Gnupylot; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
Created on 04/feb/2010

@author: Lorenzo Rovigatti
'''

from PyQt4 import QtCore, QtGui
try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO
import numpy

try:
    from Gnuplot import PlotItems, utils
except ImportError:
    from src.Gnuplot import PlotItems, utils

class PlotObject():
    '''
    A PlotObject is a generic object that can be plotted using Gnupylot. It uses, internally, 
    a PlotItem to store the item to be plotted but it's something more: it can be converted 
    to a string to be saved, it can be loaded and it joins everything Gnupylot needs to manage 
    PlotItems.
    '''


    def __init__(self, name):
        '''
        Constructor
        '''
        self.__name = name
        self.__hidden = False
        self.plotItem = None
        """PlotItem stored."""

    def getName(self):
        return self.__name

    def setName(self, value):
        self.__name = value

    def getHiddenStatus(self):
        return self.__hidden

    def setHiddenStatus(self, value):
        self.__hidden = value

    def isParameter(self, par):
        return self.parameters.has_key(par)

    def getParameter(self, par):
        return self.parameters[par]

    def getParameters(self):
        return self.__parameters

    def setParameters(self, newParams):
        self.__parameters = {}
        self.mergeParameters(newParams)

    def mergeParameters(self, newParams):
        for k, v in newParams.iteritems():
            if k == "with_":
                tokens = v.split()
                self.__parameters["with_"] = tokens[0]
                for i in range(1, len(tokens), 2):
                    self.__parameters[tokens[i]] = tokens[i + 1]
            else: self.__parameters[k] = v

    def getPlotItem(self):
        return self.plotItem

    name = property(getName, setName, "Name of the PlotObject.")
    parameters = property(getParameters, setParameters, "Function parameters (with_, linetype, etc.).")
    hidden = property(getHiddenStatus, setHiddenStatus, "Is this object hidden?")


class Label(PlotObject):
    '''
    Incapsulates a gnuplot label. Here the name is the gnuplot's 'tag'.
    '''
    def __init__(self, name, text, position=None, font=None, **keyw):
        PlotObject.__init__(self, name)

        self.text = text
        self.position = position
        '''
        This tuple contains x position, y position, alignment (left|center|right), origin of reference frame (first|second|graph|screen)
        @type: tuple
        '''
        self.font = font

    def getText(self):
        return self.__text

    def setText(self, value):
        self.__text = value

    def getPlotItem(self):
        '''Returns the string that will be used in gnuplot.'''
        toRet = 'set label %s "%s"' % (self.name, self.text)
        if self.position != None:
            toRet += ' at %s %s' % (self.position['origin'], self.position['x'])
            if self.position['y'] != None:
                toRet += ',%s' % self.position['y']
            toRet += ' %s' % self.position['relative']

        if self.font != None:
            toRet += ' font "%s"' % self.font

        return toRet

    text = property(getText, setText, "Label's text.")


class Function(PlotObject):
    def __init__(self, name, definition, **keyw):
        '''
        Constructor
        @param name: name of the function
        @type name: string
        @param definition: definition of the function (for example sin(x))
        @type definition: string
        '''
        PlotObject.__init__(self, name)

        self.plotItem = PlotItems.Func(name, **keyw)
        """Stored L{PlotItems.Func}
        @type: L{Func<PlotItems.Func>}"""

        self.definition = definition
        self.setParameters(keyw)

    def getDefinition(self):
        return self.__definition

    def setDefinition(self, value):
        self.__definition = value

    definition = property(getDefinition, setDefinition, "Function definition.")


class Data(PlotObject, QtCore.QAbstractTableModel):
    '''
    Incapsulates a gnuplot dataset.
    '''
    def __init__(self, name, data, **keyw):
        '''
        Constructor
        '''
        PlotObject.__init__(self, name)
        QtCore.QAbstractTableModel.__init__(self)

        # reproduce the PlotItems.Data function
        data = utils.float_array(data)
        if "cols" in keyw:
            data = numpy.take(data, keyw['cols'], -1)
            del keyw['cols']

        self.keyw = keyw
        """Options to be passed to the _NewFileItem constructor.
        @type: dictionary"""
        self.mydata = data
        """Raw stored data
        @type: array"""

        self.parameters = keyw

        self.updateFileItem()

    def updateFileItem(self):
        """Updates the stored PlotItem using the raw stored data as a source."""
        f = StringIO()
        utils.write_array(f, self.mydata)
        content = f.getvalue()
        self.plotItem = PlotItems._NewFileItem(content, **self.keyw)
        """Stored L{PlotItems._NewFileItem}
        @type: L{_NewFileItem<PlotItems._NewFileItem>}"""

    def columnCount(self, parent=QtCore.QModelIndex()):
        return len(self.mydata[0])

    def rowCount(self, parent=QtCore.QModelIndex()):
        return len(self.mydata)

    def data(self, index, role=QtCore.Qt.DisplayRole):
        if role == QtCore.Qt.DisplayRole or role == QtCore.Qt.EditRole:
            return str(self.mydata[index.row(), index.column()])

        return QtCore.QVariant()

    def headerData(self, section, orientation, role=QtCore.Qt.DisplayRole):
        if role != QtCore.Qt.DisplayRole:
            return QtCore.QVariant()

        if orientation == QtCore.Qt.Horizontal:
            if section == 0: return "x"
            elif section == 1: return "y"
            else: return "err"
        else: return str(section + 1)

        return QtCore.QVariant()

    def flags(self, index):
        if not index.isValid():
            return QtCore.Qt.ItemIsEnabled

        return QtCore.QAbstractTableModel.flags(self, index) | QtCore.Qt.ItemIsEditable

    def setData(self, index, value, role):
        if index.isValid() and role == QtCore.Qt.EditRole:
            self.mydata[index.row(), index.column()] = value
            self.emit(QtCore.SIGNAL('dataChanged(const QModelIndex &, const QModelIndex &)'), index, index)
            return True

        return False

    def insertRow(self, row):
        """Insert a new row in the dataset at the given position.
        The new row will have items with default value 0."""
        self.beginInsertRows(QtCore.QModelIndex(), row, row)

        emptyRow = [0, ] * self.mydata.shape[1]
        self.mydata = numpy.insert(self.mydata, row, emptyRow, axis=0)

        self.endInsertRows()
        return True

    def insertRows(self, row, count):
        """Not Implemented"""
        pass
#        self.beginInsertRows(QtCore.QModelIndex(), row, row + count - 1)
#        
#        for i in range(0, count):
#            self.objects.insert(row, None)
#        
#        self.endInsertRows()
#        return True

    def removeRow(self, row):
        """Remove a single row from the model."""
        self.beginRemoveRows(QtCore.QModelIndex(), row, row)

        self.mydata = numpy.delete(self.mydata, row, axis=0)

        self.endRemoveRows()
        return True

    def removeSparseRows(self, rows):
        """
        Remove all the given rows.
        @param rows: list of rows to be removed. It must be sorted
        @type rows: list
        """
        self.beginRemoveRows(QtCore.QModelIndex(), rows[0], rows[len(rows) - 1])

        for row in rows:
            self.mydata = numpy.delete(self.mydata, row, axis=0)

        self.endRemoveRows()

    def removeRows(self, row, count):
        """Not implemented."""
        pass
#        self.beginRemoveRows(QtCore.QModelIndex(), row, row + count - 1)
#        
#        for i in range(0, count):
#            self.mydata = numpy.delete(self.mydata, row, axis=0)
#        
#        self.endRemoveRows()
#        return True

    def __deepcopy__(self, memo={}):
        """Perform a deepcopy of this object.
        Reimplemented for necessity: I couldn't manage to deepcopy this object normally."""
        return Data(self.name, self.mydata, **self.keyw)


class DataDelegate(QtGui.QItemDelegate):
    """ItemDelegate used to modify datasets using a TableView."""
    def __init__(self):
        QtGui.QItemDelegate.__init__(self)
        self.validator = QtGui.QDoubleValidator(self)
        """L{QDoubleValidator} used with editors
        @type: L{QDoubleValidator}"""

    def createEditor(self, parent, option, index):
        editor = QtGui.QLineEdit(parent)
        editor.setValidator(self.validator)

        return editor

    def setEditorData(self, editor, index):
        val = str(index.model().data(index, QtCore.Qt.EditRole))
        editor.setText(val)

    def setModelData(self, editor, model, index):
        val = editor.text()

        model.setData(index, val, QtCore.Qt.EditRole)

    def updateEditorGeometry(self, editor, option, index):
        editor.setGeometry(option.rect)
