#   This file is part of Gnupylot.
#
#    Gnupylot is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Gnupylot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Gnupylot; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
Created on 09/feb/2010

@author: Lorenzo Rovigatti
'''

from PyQt4 import QtGui
from copy import deepcopy

class UndoCommand(QtGui.QUndoCommand):
    """Basic undo command.
    All the other commands must inherit from this class."""
    def __init__(self):
        QtGui.QUndoCommand.__init__(self)


class ToggleHiddenStatus(UndoCommand):
    def __init__(self, objects, index):
        UndoCommand.__init__(self)

        self.objects = objects
        self.index = index
        self.oldStatus = objects[index].getHiddenStatus()

    def redo(self):
        self.objects[self.index].setHiddenStatus(not self.oldStatus)

    def undo(self):
        self.objects[self.index].setHiddenStatus(self.oldStatus)


# Label commands
class AddLabelCommand(UndoCommand):
    """This command add a label to the actual plot."""
    def __init__(self, labels, args):
        UndoCommand.__init__(self)
        self.labels = labels
        self.args = args
        self.setText("add label '%s'" % self.args['text'])

    def redo(self):
        self.index = self.labels.appendLabel(**self.args)

    def undo(self):
        self.labels.removeItem(self.index)


class EditLabelCommand(UndoCommand):
    """This command edit the selected label of the actual plot."""
    def __init__(self, labels, index, args):
        UndoCommand.__init__(self)
        self.labels = labels
        self.oldLabel = deepcopy(self.labels[index])
        self.index = index
        self.args = args
        self.setText("edit label '%s'" % self.args['text'])

    def redo(self):
        self.labels.editLabel(self.index, **self.args)

    def undo(self):
        self.labels.setData(self.index, self.oldLabel)


class DelLabelCommand(UndoCommand):
    """This command remove the selected label from the actual plot."""
    def __init__(self, labels, index):
        UndoCommand.__init__(self)
        self.labels = labels
        self.oldLabel = deepcopy(self.labels[index])
        self.index = index
        self.setText("del label '%s'" % self.oldLabel.text)

    def redo(self):
        self.labels.removeItem(self.index)

    def undo(self):
        self.labels.insertData(self.index, self.oldLabel)


# Function commands        
class AddFunctionCommand(UndoCommand):
    """This command add a function to the actual plot."""
    def __init__(self, functions, args):
        UndoCommand.__init__(self)
        self.functions = functions
        self.args = args
        self.setText("add function '%s'" % self.args['name'])

    def redo(self):
        self.index = self.functions.appendFunc(**self.args)

    def undo(self):
        self.functions.removeItem(self.index)


class EditFunctionCommand(UndoCommand):
    """This command edit the selected function of the actual plot."""
    def __init__(self, functions, index, args):
        UndoCommand.__init__(self)
        self.functions = functions
        self.oldFunc = deepcopy(self.functions[index])
        self.index = index
        self.args = args
        self.setText("edit function '%s'" % self.args['name'])

    def redo(self):
        self.functions.editFunc(self.index, **self.args)

    def undo(self):
        self.functions.setData(self.index, self.oldFunc)


class DelFunctionCommand(UndoCommand):
    """This command remove the selected function from the actual plot."""
    def __init__(self, functions, index):
        UndoCommand.__init__(self)
        self.functions = functions
        self.oldFunc = deepcopy(self.functions[index])
        self.index = index
        self.setText("del function '%s'" % self.oldFunc.name)

    def redo(self):
        self.functions.removeItem(self.index)

    def undo(self):
        self.functions.insertData(self.index, self.oldFunc)


# Dataset commands
class AddDatasetCommand(UndoCommand):
    """This command adds a dataset to the actual plot."""
    def __init__(self, datasets, args):
        UndoCommand.__init__(self)
        self.datasets = datasets
        self.args = args
        self.setText("add dataset '%s'" % self.args['name'])

    def redo(self):
        self.index = self.datasets.appendData(**self.args)

    def undo(self):
        self.datasets.removeItem(self.index)


class EditDatasetCommand(UndoCommand):
    """This command edits the data of the selected dataset of the actual plot."""
    def __init__(self, datasets, index, newDataset):
        UndoCommand.__init__(self)
        self.datasets = datasets
        self.newDataset = newDataset
        self.oldDataset = self.datasets[index]
        self.index = index
        self.setText("edit data of dataset '%s'" % self.newDataset.name)

    def redo(self):
        self.datasets.setData(self.index, self.newDataset)
        self.newDataset.updateFileItem()

    def undo(self):
        self.datasets.setData(self.index, self.oldDataset)
        self.oldDataset.updateFileItem()


class EditDatasetAppearanceCommand(UndoCommand):
    """This command edits the appearance of the selected dataset of the actual plot."""
    def __init__(self, datasets, index, dataset, keyw):
        UndoCommand.__init__(self)
        self.name = keyw["name"]
        del keyw["name"]
        self.keyw = keyw

        self.oldName = dataset.name
        self.oldKeyw = deepcopy(dataset.keyw)

        self.dataset = dataset
        self.datasets = datasets
        self.oldDataset = self.datasets[index]
        self.index = index
        self.setText("edit appearance of dataset '%s'" % self.dataset.name)

    def redo(self):
        self.dataset.name = self.name
        self.dataset.keyw['title'] = self.keyw['title']
        self.dataset.keyw['with_'] = self.keyw['with_']
        self.datasets.setData(self.index, self.dataset)
        self.dataset.updateFileItem()

    def undo(self):
        self.dataset.name = self.oldName
        self.dataset.keyw = self.oldKeyw
        self.datasets.setData(self.index, self.dataset)
        self.dataset.updateFileItem()


class DelDatasetCommand(UndoCommand):
    """This command removes the selected dataset from the actual plot."""
    def __init__(self, datasets, index):
        UndoCommand.__init__(self)
        self.datasets = datasets
        self.oldDataset = deepcopy(self.datasets[index])
        self.index = index
        self.setText("del dataset '%s'" % self.oldDataset.name)

    def redo(self):
        self.datasets.removeItem(self.index)

    def undo(self):
        self.datasets.insertData(self.index, self.oldDataset)


class ChangePlotSettingsCommand(UndoCommand):
    """This command changes the given plot's settings."""
    def __init__(self, plot, nSettings):
        UndoCommand.__init__(self)
        self.plot = plot
        self.oldSettings = deepcopy(plot.settings)
        self.settings = nSettings

        self.setText("change plot settings")

    def redo(self):
        self.plot.settings = self.settings

    def undo(self):
        self.plot.settings = self.oldSettings
